/* -*-c-*-
 *
 * Generalized exponentiation
 *
 * (c) 2001 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Header files ------------------------------------------------------*/

#define EXP_TYPE /* Hack */
#include "exp.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @exp_simulnext@ --- *
 *
 * Arguments:	@exp_simul *e@ = pointer to state structure
 *		@size_t x@ = a current accumulator
 *
 * Returns:	The next column of bits.
 *
 * Use:		Scans the next column of bits for a simultaneous
 *		exponentiation.
 */

size_t exp_simulnext(exp_simul *e, size_t x)
{
  size_t i;

  /* --- Move to the next word along --- */

  if (!e->b) {
    e->o--;
    for (i = 0; i < e->n; i++)
      e->s[i].w = e->o < e->s[i].len ? e->s[i].v[e->o] : 0;
    e->b = MPW_BITS;
  }

  /* --- Scan out a column of bits --- */

  for (i = 0; i < e->n; i++) {
    x = (x << 1) | ((e->s[i].w >> (MPW_BITS - 1)) & 1u);
    e->s[i].w <<= 1;
  }
  e->b--;
  return (x);
}

/*----- That's all, folks -------------------------------------------------*/
